/*
 * analogfunction.cpp - analog function implementations
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 */


#include <stdio.h>
#include "analogfunction.h"
#include <math.h>

#ifdef __cplusplus
extern "C"
{
#endif /*__cplusplus*/
inline double _cos(double arg)             { return  cos(arg); }
inline double _d0_cos(double arg)          { return (-sin(arg)); }
inline double _sin(double arg)             { return  sin(arg); }
inline double _d0_sin(double arg)          { return (cos(arg)); }
inline double _tan(double arg)             { return  tan(arg); }
inline double _d0_tan(double arg)          { return (1.0/cos(arg)/cos(arg)); }
inline double _cosh(double arg)            { return  cosh(arg); }
inline double _d0_cosh(double arg)         { return (sinh(arg)); }
inline double _sinh(double arg)            { return  sinh(arg); }
inline double _d0_sinh(double arg)         { return (cosh(arg)); }
inline double _tanh(double arg)            { return  tanh(arg); }
inline double _d0_tanh(double arg)         { return (1.0/cosh(arg)/cosh(arg)); }
inline double _acos(double arg)            { return  acos(arg); }
inline double _d0_acos(double arg)         { return (-1.0/sqrt(1-arg*arg)); }
inline double _asin(double arg)            { return  asin(arg); }
inline double _d0_asin(double arg)         { return (+1.0/sqrt(1-arg*arg)); }
inline double _atan(double arg)            { return  atan(arg); }
inline double _d0_atan(double arg)         { return (+1.0/(1+arg*arg)); }
inline double _logE(double arg)            { return  log(arg); }
inline double _d0_logE(double arg)         { return (1.0/arg); }
inline double _log10(double arg)           { return  log10(arg); }
inline double _d0_log10(double arg)        { return (1.0/arg/log((double)10)); }
inline double _exp(double arg)             { return  exp(arg); }
inline double _d0_exp(double arg)          { return exp(arg); } /* fixme */
inline double _sqrt(double arg)            { return  sqrt(arg); }
inline double _d0_sqrt(double arg)         { return (1.0/sqrt(arg)/2.0); } /* fixme */
inline double _abs(double arg)             { return  fabs(arg); }
inline double _d0_abs(double arg)          { return (((arg)>=0)?(+1.0):(-1.0)); }

inline double _hypot(double x,double y)    { return sqrt((x)*(x)+(y)*(y)); }
inline double _d0_hypot(double x,double y) { return (x)/sqrt((x)*(x)+(y)*(y)); } /* fixme */
inline double _d1_hypot(double x,double y) { return (y)/sqrt((x)*(x)+(y)*(y)); } /* fixme */

inline double _max(double x,double y)      { return ((x)>(y))?(x):(y); }
inline double _d0_max(double x,double y)   { return ((x)>(y))?1.0:0.0; }
inline double _d1_max(double x,double y)   { return ((x)>(y))?0.0:1.0; }

inline double _min(double x,double y)      { return ((x)<(y))?(x):(y); }
inline double _d0_min(double x,double y)   { return ((x)<(y))?1.0:0.0; }
inline double _d1_min(double x,double y)   { return ((x)<(y))?0.0:1.0; }

inline double _pow(double x,double y)      { return pow(x,y); }
inline double _d0_pow(double x,double y)   { return (x==0.0)?0.0:((y/x)*pow(x,y)); } /* fixme */
inline double _d1_pow(double x,double y)   { return (x==0.0)?0.0:((log(x)/exp(0.0))*pow(x,y)); } /* fixme */

#define _limexp(arg)    _exp(val,arg)
#define _d_limexp(arg)  _d_exp(val,dval,arg)

inline double cap_vt (double Tdev) {
  return 1.3806503e-23*Tdev/1.602176462e-19;
}
inline double cap_d_vt (double Tdev, double d_Tdev) {
  double vt_Tdev=1.3806503e-23/1.602176462e-19;
  return vt_Tdev*d_Tdev;
}

#ifdef STAND_ALONE
int
main(void)
{
  printf("(1.0,1.0) norm0=%e dnorm0=%e [expected: 1.0]\n",          cap_norm(1.0,1.0,0.0), cap_d_norm(1.0,1.0,0.0,1.0,1.0,1.0));
  printf("(1.0,1.0) norm1=%e dnorm1=%e [expected: 2.0]\n",          cap_norm(1.0,1.0,1.0), cap_d_norm(1.0,1.0,1.0,1.0,1.0,1.0));
  printf("(1.0,1.0) norm2=%e dnorm2=%e [expected: sqrt(2.0)=%e]\n",cap_norm(1.0,1.0,2.0), cap_d_norm(1.0,1.0,2.0,1.0,1.0,1.0),sqrt(2));
  double x,xnb=500.0,xmin=0.0,xmax=10.0,xstep=(xmax-xmin)/xnb;
  FILE*y=fopen("y.data","w");
  FILE*dy=fopen("dy.data","w");
  for(x=xmin;x<=xmax;x+=xstep)
  {
    fprintf(y,"%.2f %.2f\n",x,cap_(x));
    fprintf(dy,"%.2f %.2f\n",x,cap_d_(x,1.0));
  }
  fclose(y);
  fclose(dy);
  return 0;
}
#endif /* STAND_ALONE */

#ifdef __cplusplus
}
#endif /*__cplusplus*/
